@@ -5,84 +5,94 @@ Sample usage:
 
 >>> from django.utils import feedgenerator
 >>> feed = feedgenerator.Rss201rev2Feed(
-...     title=u"Poynter E-Media Tidbits",
-...     link=u"http://www.poynter.org/column.asp?id=31",
-...     description=u"A group Weblog by the sharpest minds in online media/journalism/publishing.",
-...     language=u"en",
+...     title="Poynter E-Media Tidbits",
+...     link="http://www.poynter.org/column.asp?id=31",
+...     description="A group Weblog by the sharpest minds in online media/journalism/publishing.",
+...     language="en",
 ... )
 >>> feed.add_item(
 ...     title="Hello",
-...     link=u"http://www.holovaty.com/test/",
+...     link="http://www.holovaty.com/test/",
 ...     description="Testing."
 ... )
->>> fp = open('test.rss', 'w')
->>> feed.write(fp, 'utf-8')
->>> fp.close()
+>>> with open('test.rss', 'w') as fp:
+...     feed.write(fp, 'utf-8')
 
 For definitions of the different versions of RSS, see:
-http://diveintomark.org/archives/2004/02/04/incompatible-rss
+http://web.archive.org/web/20110718035220/http://diveintomark.org/archives/2004/02/04/incompatible-rss
 """
+from __future__ import unicode_literals
 
 import datetime
-import urlparse
+try:
+    from urllib.parse import urlparse
+except ImportError:     # Python 2
+    from urlparse import urlparse
 from django.utils.xmlutils import SimplerXMLGenerator
-from django.utils.encoding import force_unicode, iri_to_uri
+from django.utils.encoding import force_text, iri_to_uri
 from django.utils import datetime_safe
+from django.utils import six
+from django.utils.six import StringIO
+from django.utils.timezone import is_aware
 
 def rfc2822_date(date):
+    # We can't use strftime() because it produces locale-dependant results, so
+    # we have to map english month and day names manually
+    months = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec',)
+    days = ('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun')
     # Support datetime objects older than 1900
     date = datetime_safe.new_datetime(date)
     # We do this ourselves to be timezone aware, email.Utils is not tz aware.
-    if date.tzinfo:
-        time_str = date.strftime('%a, %d %b %Y %H:%M:%S ')
+    dow = days[date.weekday()]
+    month = months[date.month - 1]
+    time_str = date.strftime('%s, %%d %s %%Y %%H:%%M:%%S ' % (dow, month))
+    if not six.PY3:             # strftime returns a byte string in Python 2
+        time_str = time_str.decode('utf-8')
+    if is_aware(date):
         offset = date.tzinfo.utcoffset(date)
-        timezone = (offset.days * 24 * 60) + (offset.seconds / 60)
+        timezone = (offset.days * 24 * 60) + (offset.seconds // 60)
         hour, minute = divmod(timezone, 60)
-        return time_str + "%+03d%02d" % (hour, minute)
+        return time_str + '%+03d%02d' % (hour, minute)
     else:
-        return date.strftime('%a, %d %b %Y %H:%M:%S -0000')
+        return time_str + '-0000'
 
 def rfc3339_date(date):
     # Support datetime objects older than 1900
     date = datetime_safe.new_datetime(date)
-    if date.tzinfo:
-        time_str = date.strftime('%Y-%m-%dT%H:%M:%S')
+    time_str = date.strftime('%Y-%m-%dT%H:%M:%S')
+    if not six.PY3:             # strftime returns a byte string in Python 2
+        time_str = time_str.decode('utf-8')
+    if is_aware(date):
         offset = date.tzinfo.utcoffset(date)
-        timezone = (offset.days * 24 * 60) + (offset.seconds / 60)
+        timezone = (offset.days * 24 * 60) + (offset.seconds // 60)
         hour, minute = divmod(timezone, 60)
-        return time_str + "%+03d:%02d" % (hour, minute)
+        return time_str + '%+03d:%02d' % (hour, minute)
     else:
-        return date.strftime('%Y-%m-%dT%H:%M:%SZ')
+        return time_str + 'Z'
 
 def get_tag_uri(url, date):
     """
     Creates a TagURI.
 
-    See http://diveintomark.org/archives/2004/05/28/howto-atom-id
+    See http://web.archive.org/web/20110514113830/http://diveintomark.org/archives/2004/05/28/howto-atom-id
     """
-    url_split = urlparse.urlparse(url)
-
-    # Python 2.4 didn't have named attributes on split results or the hostname.
-    hostname = getattr(url_split, 'hostname', url_split[1].split(':')[0])
-    path = url_split[2]
-    fragment = url_split[5]
-
+    bits = urlparse(url)
     d = ''
     if date is not None:
         d = ',%s' % datetime_safe.new_datetime(date).strftime('%Y-%m-%d')
-    return u'tag:%s%s:%s/%s' % (hostname, d, path, fragment)
+    return 'tag:%s%s:%s/%s' % (bits.hostname, d, bits.path, bits.fragment)
 
 class SyndicationFeed(object):
     "Base class for all syndication feeds. Subclasses should provide write()"
     def __init__(self, title, link, description, language=None, author_email=None,
             author_name=None, author_link=None, subtitle=None, categories=None,
             feed_url=None, feed_copyright=None, feed_guid=None, ttl=None, **kwargs):
-        to_unicode = lambda s: force_unicode(s, strings_only=True)
+        to_unicode = lambda s: force_text(s, strings_only=True)
         if categories:
-            categories = [force_unicode(c) for c in categories]
+            categories = [force_text(c) for c in categories]
         if ttl is not None:
             # Force ints to unicode
-            ttl = force_unicode(ttl)
+            ttl = force_text(ttl)
         self.feed = {
             'title': to_unicode(title),
             'link': iri_to_uri(link),
@@ -110,12 +120,12 @@ class SyndicationFeed(object):
         objects except pubdate, which is a datetime.datetime object, and
         enclosure, which is an instance of the Enclosure class.
         """
-        to_unicode = lambda s: force_unicode(s, strings_only=True)
+        to_unicode = lambda s: force_text(s, strings_only=True)
         if categories:
             categories = [to_unicode(c) for c in categories]
         if ttl is not None:
             # Force ints to unicode
-            ttl = force_unicode(ttl)
+            ttl = force_text(ttl)
         item = {
             'title': to_unicode(title),
             'link': iri_to_uri(link),
@@ -174,7 +184,6 @@ class SyndicationFeed(object):
         """
         Returns the feed in the given encoding as a string.
         """
-        from StringIO import StringIO
         s = StringIO()
         self.write(s, encoding)
         return s.getvalue()
@@ -199,179 +208,181 @@ class Enclosure(object):
         self.url = iri_to_uri(url)
 
 class RssFeed(SyndicationFeed):
-    mime_type = 'application/rss+xml'
+    mime_type = 'application/rss+xml; charset=utf-8'
     def write(self, outfile, encoding):
         handler = SimplerXMLGenerator(outfile, encoding)
         handler.startDocument()
-        handler.startElement(u"rss", self.rss_attributes())
-        handler.startElement(u"channel", self.root_attributes())
+        handler.startElement("rss", self.rss_attributes())
+        handler.startElement("channel", self.root_attributes())
         self.add_root_elements(handler)
         self.write_items(handler)
         self.endChannelElement(handler)
-        handler.endElement(u"rss")
+        handler.endElement("rss")
 
     def rss_attributes(self):
-        return {u"version": self._version,
-                u"xmlns:atom": u"http://www.w3.org/2005/Atom"}
+        return {"version": self._version,
+                "xmlns:atom": "http://www.w3.org/2005/Atom"}
 
     def write_items(self, handler):
         for item in self.items:
-            handler.startElement(u'item', self.item_attributes(item))
+            handler.startElement('item', self.item_attributes(item))
             self.add_item_elements(handler, item)
-            handler.endElement(u"item")
+            handler.endElement("item")
 
     def add_root_elements(self, handler):
-        handler.addQuickElement(u"title", self.feed['title'])
-        handler.addQuickElement(u"link", self.feed['link'])
-        handler.addQuickElement(u"description", self.feed['description'])
-        handler.addQuickElement(u"atom:link", None, {u"rel": u"self", u"href": self.feed['feed_url']})
+        handler.addQuickElement("title", self.feed['title'])
+        handler.addQuickElement("link", self.feed['link'])
+        handler.addQuickElement("description", self.feed['description'])
+        if self.feed['feed_url'] is not None:
+            handler.addQuickElement("atom:link", None,
+                    {"rel": "self", "href": self.feed['feed_url']})
         if self.feed['language'] is not None:
-            handler.addQuickElement(u"language", self.feed['language'])
+            handler.addQuickElement("language", self.feed['language'])
         for cat in self.feed['categories']:
-            handler.addQuickElement(u"category", cat)
+            handler.addQuickElement("category", cat)
         if self.feed['feed_copyright'] is not None:
-            handler.addQuickElement(u"copyright", self.feed['feed_copyright'])
-        handler.addQuickElement(u"lastBuildDate", rfc2822_date(self.latest_post_date()).decode('utf-8'))
+            handler.addQuickElement("copyright", self.feed['feed_copyright'])
+        handler.addQuickElement("lastBuildDate", rfc2822_date(self.latest_post_date()))
         if self.feed['ttl'] is not None:
-            handler.addQuickElement(u"ttl", self.feed['ttl'])
+            handler.addQuickElement("ttl", self.feed['ttl'])
 
     def endChannelElement(self, handler):
-        handler.endElement(u"channel")
+        handler.endElement("channel")
 
 class RssUserland091Feed(RssFeed):
-    _version = u"0.91"
+    _version = "0.91"
     def add_item_elements(self, handler, item):
-        handler.addQuickElement(u"title", item['title'])
-        handler.addQuickElement(u"link", item['link'])
+        handler.addQuickElement("title", item['title'])
+        handler.addQuickElement("link", item['link'])
         if item['description'] is not None:
-            handler.addQuickElement(u"description", item['description'])
+            handler.addQuickElement("description", item['description'])
 
 class Rss201rev2Feed(RssFeed):
     # Spec: http://blogs.law.harvard.edu/tech/rss
-    _version = u"2.0"
+    _version = "2.0"
     def add_item_elements(self, handler, item):
-        handler.addQuickElement(u"title", item['title'])
-        handler.addQuickElement(u"link", item['link'])
+        handler.addQuickElement("title", item['title'])
+        handler.addQuickElement("link", item['link'])
         if item['description'] is not None:
-            handler.addQuickElement(u"description", item['description'])
+            handler.addQuickElement("description", item['description'])
 
         # Author information.
         if item["author_name"] and item["author_email"]:
-            handler.addQuickElement(u"author", "%s (%s)" % \
+            handler.addQuickElement("author", "%s (%s)" % \
                 (item['author_email'], item['author_name']))
         elif item["author_email"]:
-            handler.addQuickElement(u"author", item["author_email"])
+            handler.addQuickElement("author", item["author_email"])
         elif item["author_name"]:
-            handler.addQuickElement(u"dc:creator", item["author_name"], {u"xmlns:dc": u"http://purl.org/dc/elements/1.1/"})
+            handler.addQuickElement("dc:creator", item["author_name"], {"xmlns:dc": "http://purl.org/dc/elements/1.1/"})
 
         if item['pubdate'] is not None:
-            handler.addQuickElement(u"pubDate", rfc2822_date(item['pubdate']).decode('utf-8'))
+            handler.addQuickElement("pubDate", rfc2822_date(item['pubdate']))
         if item['comments'] is not None:
-            handler.addQuickElement(u"comments", item['comments'])
+            handler.addQuickElement("comments", item['comments'])
         if item['unique_id'] is not None:
-            handler.addQuickElement(u"guid", item['unique_id'])
+            handler.addQuickElement("guid", item['unique_id'])
         if item['ttl'] is not None:
-            handler.addQuickElement(u"ttl", item['ttl'])
+            handler.addQuickElement("ttl", item['ttl'])
 
         # Enclosure.
         if item['enclosure'] is not None:
-            handler.addQuickElement(u"enclosure", '',
-                {u"url": item['enclosure'].url, u"length": item['enclosure'].length,
-                    u"type": item['enclosure'].mime_type})
+            handler.addQuickElement("enclosure", '',
+                {"url": item['enclosure'].url, "length": item['enclosure'].length,
+                    "type": item['enclosure'].mime_type})
 
         # Categories.
         for cat in item['categories']:
-            handler.addQuickElement(u"category", cat)
+            handler.addQuickElement("category", cat)
 
 class Atom1Feed(SyndicationFeed):
     # Spec: http://atompub.org/2005/07/11/draft-ietf-atompub-format-10.html
-    mime_type = 'application/atom+xml; charset=utf8'
-    ns = u"http://www.w3.org/2005/Atom"
+    mime_type = 'application/atom+xml; charset=utf-8'
+    ns = "http://www.w3.org/2005/Atom"
 
     def write(self, outfile, encoding):
         handler = SimplerXMLGenerator(outfile, encoding)
         handler.startDocument()
-        handler.startElement(u'feed', self.root_attributes())
+        handler.startElement('feed', self.root_attributes())
         self.add_root_elements(handler)
         self.write_items(handler)
-        handler.endElement(u"feed")
+        handler.endElement("feed")
 
     def root_attributes(self):
         if self.feed['language'] is not None:
-            return {u"xmlns": self.ns, u"xml:lang": self.feed['language']}
+            return {"xmlns": self.ns, "xml:lang": self.feed['language']}
         else:
-            return {u"xmlns": self.ns}
+            return {"xmlns": self.ns}
 
     def add_root_elements(self, handler):
-        handler.addQuickElement(u"title", self.feed['title'])
-        handler.addQuickElement(u"link", "", {u"rel": u"alternate", u"href": self.feed['link']})
+        handler.addQuickElement("title", self.feed['title'])
+        handler.addQuickElement("link", "", {"rel": "alternate", "href": self.feed['link']})
         if self.feed['feed_url'] is not None:
-            handler.addQuickElement(u"link", "", {u"rel": u"self", u"href": self.feed['feed_url']})
-        handler.addQuickElement(u"id", self.feed['id'])
-        handler.addQuickElement(u"updated", rfc3339_date(self.latest_post_date()).decode('utf-8'))
+            handler.addQuickElement("link", "", {"rel": "self", "href": self.feed['feed_url']})
+        handler.addQuickElement("id", self.feed['id'])
+        handler.addQuickElement("updated", rfc3339_date(self.latest_post_date()))
         if self.feed['author_name'] is not None:
-            handler.startElement(u"author", {})
-            handler.addQuickElement(u"name", self.feed['author_name'])
+            handler.startElement("author", {})
+            handler.addQuickElement("name", self.feed['author_name'])
             if self.feed['author_email'] is not None:
-                handler.addQuickElement(u"email", self.feed['author_email'])
+                handler.addQuickElement("email", self.feed['author_email'])
             if self.feed['author_link'] is not None:
-                handler.addQuickElement(u"uri", self.feed['author_link'])
-            handler.endElement(u"author")
+                handler.addQuickElement("uri", self.feed['author_link'])
+            handler.endElement("author")
         if self.feed['subtitle'] is not None:
-            handler.addQuickElement(u"subtitle", self.feed['subtitle'])
+            handler.addQuickElement("subtitle", self.feed['subtitle'])
         for cat in self.feed['categories']:
-            handler.addQuickElement(u"category", "", {u"term": cat})
+            handler.addQuickElement("category", "", {"term": cat})
         if self.feed['feed_copyright'] is not None:
-            handler.addQuickElement(u"rights", self.feed['feed_copyright'])
+            handler.addQuickElement("rights", self.feed['feed_copyright'])
 
     def write_items(self, handler):
         for item in self.items:
-            handler.startElement(u"entry", self.item_attributes(item))
+            handler.startElement("entry", self.item_attributes(item))
             self.add_item_elements(handler, item)
-            handler.endElement(u"entry")
+            handler.endElement("entry")
 
     def add_item_elements(self, handler, item):
-        handler.addQuickElement(u"title", item['title'])
-        handler.addQuickElement(u"link", u"", {u"href": item['link'], u"rel": u"alternate"})
+        handler.addQuickElement("title", item['title'])
+        handler.addQuickElement("link", "", {"href": item['link'], "rel": "alternate"})
         if item['pubdate'] is not None:
-            handler.addQuickElement(u"updated", rfc3339_date(item['pubdate']).decode('utf-8'))
+            handler.addQuickElement("updated", rfc3339_date(item['pubdate']))
 
         # Author information.
         if item['author_name'] is not None:
-            handler.startElement(u"author", {})
-            handler.addQuickElement(u"name", item['author_name'])
+            handler.startElement("author", {})
+            handler.addQuickElement("name", item['author_name'])
             if item['author_email'] is not None:
-                handler.addQuickElement(u"email", item['author_email'])
+                handler.addQuickElement("email", item['author_email'])
             if item['author_link'] is not None:
-                handler.addQuickElement(u"uri", item['author_link'])
-            handler.endElement(u"author")
+                handler.addQuickElement("uri", item['author_link'])
+            handler.endElement("author")
 
         # Unique ID.
         if item['unique_id'] is not None:
             unique_id = item['unique_id']
         else:
             unique_id = get_tag_uri(item['link'], item['pubdate'])
-        handler.addQuickElement(u"id", unique_id)
+        handler.addQuickElement("id", unique_id)
 
         # Summary.
         if item['description'] is not None:
-            handler.addQuickElement(u"summary", item['description'], {u"type": u"html"})
+            handler.addQuickElement("summary", item['description'], {"type": "html"})
 
         # Enclosure.
         if item['enclosure'] is not None:
-            handler.addQuickElement(u"link", '',
-                {u"rel": u"enclosure",
-                 u"href": item['enclosure'].url,
-                 u"length": item['enclosure'].length,
-                 u"type": item['enclosure'].mime_type})
+            handler.addQuickElement("link", '',
+                {"rel": "enclosure",
+                 "href": item['enclosure'].url,
+                 "length": item['enclosure'].length,
+                 "type": item['enclosure'].mime_type})
 
         # Categories.
         for cat in item['categories']:
-            handler.addQuickElement(u"category", u"", {u"term": cat})
+            handler.addQuickElement("category", "", {"term": cat})
 
         # Rights.
         if item['item_copyright'] is not None:
-            handler.addQuickElement(u"rights", item['item_copyright'])
+            handler.addQuickElement("rights", item['item_copyright'])
 
 # This isolates the decision of what the system default is, so calling code can
 # do "feedgenerator.DefaultFeed" instead of "feedgenerator.Rss201rev2Feed".
